svn commit: r190387 - head/sys/kern

John Baldwin jhb at FreeBSD.org
Tue Mar 24 11:16:45 PDT 2009


Author: jhb
Date: Tue Mar 24 18:16:42 2009
New Revision: 190387
URL: http://svn.freebsd.org/changeset/base/190387

Log:
  When a file lookup fails due to encountering a doomed vnode from a forced
  unmount, consistently return ENOENT rather than EBADF.
  
  Reviewed by:	kib
  MFC after:	1 month

Modified:
  head/sys/kern/vfs_cache.c
  head/sys/kern/vfs_lookup.c

Modified: head/sys/kern/vfs_cache.c
==============================================================================
--- head/sys/kern/vfs_lookup.c	Tue Mar 24 18:10:22 2009	(r190386)
+++ head/sys/kern/vfs_lookup.c	Tue Mar 24 18:16:42 2009	(r190387)
@@ -602,7 +602,7 @@ dirloop:
 			if ((dp->v_vflag & VV_ROOT) == 0)
 				break;
 			if (dp->v_iflag & VI_DOOMED) {	/* forced unmount */
-				error = EBADF;
+				error = ENOENT;
 				goto bad;
 			}
 			tdp = dp;
@@ -764,9 +764,11 @@ unionlookup:
 	     *ndp->ni_next == '/')) {
 		cnp->cn_flags |= ISSYMLINK;
 		if (dp->v_iflag & VI_DOOMED) {
-			/* We can't know whether the directory was mounted with
-			 * NOSYMFOLLOW, so we can't follow safely. */
-			error = EBADF;
+			/*
+			 * We can't know whether the directory was mounted with
+			 * NOSYMFOLLOW, so we can't follow safely.
+			 */
+			error = ENOENT;
 			goto bad2;
 		}
 		if (dp->v_mount->mnt_flag & MNT_NOSYMFOLLOW) {


More information about the svn-src-all mailing list
--- head/sys/kern/vfs_cache.c	Tue Mar 24 18:10:22 2009	(r190386)
+++ head/sys/kern/vfs_cache.c	Tue Mar 24 18:16:42 2009	(r190387)
@@ -320,7 +320,7 @@ cache_zap(ncp)
  * (negative cacheing), a status of ENOENT is returned. If the lookup
  * fails, a status of zero is returned.  If the directory vnode is
  * recycled out from under us due to a forced unmount, a status of
  * EBADF is returned.
  *
  * vpp is locked and ref'd on return.  If we're looking up DOTDOT, dvp is
  * unlocked.  If we're looking up . an extra ref is taken, but the lock is
@@ -472,7 +472,7 @@ success:
 					/* forced unmount */
 					vrele(*vpp);
 					*vpp = NULL;
-					return (EBADF);
+					return (ENOENT);
 				}
 			} else
 				vn_lock(*vpp, LK_DOWNGRADE | LK_RETRY);
@@ -983,7 +983,7 @@ vn_fullpath1(struct thread *td, struct v
 		if (vp->v_vflag & VV_ROOT) {
 			if (vp->v_iflag & VI_DOOMED) {	/* forced unmount */
 				CACHE_RUNLOCK();
-				error = EBADF;
+				error = ENOENT;
 				break;
 			}
 			vp = vp->v_mount->mnt_vnodecovered;
@@ -320,7 +320,7 @@ cache_zap(ncp)
  * fails, a status of zero is returned.  If the directory vnode is
  * recycled out from under us due to a forced unmount, a status of
- * EBADF is returned.
+ * ENOENT is returned.
  *
  * vpp is locked and ref'd on return.  If we're looking up DOTDOT, dvp is

Modified: head/sys/kern/vfs_lookup.c
==============================================================================